/*******************************************************************************
 * Copyright (c) 2016 ALL4TEC & CEA LIST.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     ALL4TEC & CEA LIST - initial API and implementation
 ******************************************************************************/
package org.polarsys.esf.localanalysis.execution.setup;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.transaction.RecordingCommand;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.papyrus.uml.tools.utils.PackageUtil;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.Profile;
import org.eclipse.uml2.uml.util.UMLUtil;
import org.polarsys.esf.core.profile.esfsafetyconcepts.util.ESFSafetyConceptsUtil;
import org.polarsys.esf.core.utils.ModelUtil;
import org.polarsys.esf.esfarchitectureconcepts.ISBlock;
import org.polarsys.esf.esfarchitectureconcepts.ISPort;
import org.polarsys.esf.esfarchitectureconcepts.SDirection;
import org.polarsys.esf.esflocalanalysis.ISBlockLAnalysis;
import org.polarsys.esf.esflocalanalysis.ISPortLAnalysis;
import org.polarsys.esf.esflocalanalysis.SDirectionLAnalysis;
import org.polarsys.esf.esflocalanalysis.impl.ESFLocalAnalysisPackage;
import org.polarsys.esf.esflocalanalysis.impl.SBlockLAnalysis;
import org.polarsys.esf.esflocalanalysis.impl.SLocalAnalysis;
import org.polarsys.esf.esflocalanalysis.impl.SPortLAnalysis;
import org.polarsys.esf.localanalysis.profile.set.ESFLocalAnalysisSet;


/**
 * Setup class for starting a new local analysis.
 *
 * @author $Author: ymunoz $
 * @version $Revision: 168 $
 */
public final class NewLocalAnalysisSetup {

    /** LocalAnalysis package name. */
    public static final String LOCAL_ANALYSIS_PACKAGE_NAME = "LocalAnalysis"; //$NON-NLS-1$

    /** Apply ESFLocalAnalysis profile label. */
    public static final String APPLY_ESFLOCALANALYSIS_PROFILE_LABEL = "Apply ESFLocalAnalysis profile"; //$NON-NLS-1$

    /** Create Local Analysis package label. */
    public static final String CREATE_LOCALANALYSIS_PACKAGE_LABEL = "Create LocalAnalysis package"; //$NON-NLS-1$

    /** Create SBlock_LA label. */
    public static final String CREATE_SBLOCK_LA_LABEL = "Create SBlock_LA"; //$NON-NLS-1$

    /**
     * Default constructor.
     */
    private NewLocalAnalysisSetup() {
    }

    /**
     * Setup of a new Local Analysis.
     *
     * @param pSBlock The SBlock to be analysed
     */
    public static void setup(final ISBlock pSBlock) {

        Model vESFModel = ModelUtil.getWorkingModel();
        if (vESFModel != null) {

            // Verify and apply (if necessary) ESFLocalAnalysis profile
            if (vESFModel.getAppliedProfile(ESFLocalAnalysisPackage.eNS_PREFIX) == null) {
                applyESFLocalAnalysisProfile(vESFModel);
            }

            // Retrieve 'SafetyArtifacts' package
            Package vSafetyArtifacts = ESFSafetyConceptsUtil.retrieveSafetyArtifactsPackage(vESFModel);

            if (vSafetyArtifacts != null) {
                // Retrieve 'LocalAnalysis' package
                Package vLocalAnalysisPackage = getLocalAnalysisPackage(vSafetyArtifacts);

                if (vLocalAnalysisPackage != null) {
                    if (!existLocalAnalysis(vLocalAnalysisPackage, pSBlock)) {
                        // Create SBlock_LA in 'LocalAnalysis' package
                        createSBlockLA(vLocalAnalysisPackage, pSBlock);
                    }
                }
            }
        }
    }

    /**
     * Verify if there is a LocalAnalysis of a SBlock.
     *
     * @param pLocalAnalysisPackage The LocalAnalysis package
     * @param pSBlock The SBlock to be analysed
     * @return True if exist a LocalAnalysis of a SBlock
     */
    private static boolean existLocalAnalysis(final Package pLocalAnalysisPackage, final ISBlock pSBlock) {
        Boolean vFound = false;
        List<Element> vElementsList = pLocalAnalysisPackage.getOwnedElements();
        Iterator<Element> vIterator = vElementsList.iterator();

        while (vFound == false && vIterator.hasNext()) {
            Element vElement = vIterator.next();

            if (vElement instanceof Class) {
                if (UMLUtil.getStereotypeApplication(vElement, SBlockLAnalysis.class) != null) {
                    ISBlockLAnalysis vSBlockLAnalysis =
                        (ISBlockLAnalysis) UMLUtil.getStereotypeApplication(vElement, SBlockLAnalysis.class);
                    if (vSBlockLAnalysis.getSBlock().equals(pSBlock)) {
                        vFound = true;
                    }
                }
            }
        }
        return vFound;
    }

    /**
     * Get LocalAnalysis package.
     *
     * @param pSafetyArtifacts The SafetyArtifacts package
     * @return The LocalAnalysis package
     */
    private static Package getLocalAnalysisPackage(final Package pSafetyArtifacts) {
        Package vLocalAnalysisPackage = null;
        Boolean vFound = false;
        List<Package> vPackagesList = pSafetyArtifacts.getNestedPackages();
        Iterator<Package> vIterator = vPackagesList.iterator();

        while (vFound == false && vIterator.hasNext()) {
            Package vPackage = vIterator.next();
            if (UMLUtil.getStereotypeApplication(vPackage, SLocalAnalysis.class) != null) {
                vFound = true;
                vLocalAnalysisPackage = vPackage;
            }
        }

        if (vLocalAnalysisPackage == null) {
            createLocalAnalysisPackage(pSafetyArtifacts);
            vLocalAnalysisPackage = getLocalAnalysisPackage(pSafetyArtifacts);
        }

        return vLocalAnalysisPackage;
    }

    /**
     * Create LocalAnalysis package.
     *
     * @param pSafetyArtifacts The SafetyArtifacts package
     */
    private static void createLocalAnalysisPackage(final Package pSafetyArtifacts) {
        TransactionalEditingDomain vDomain = ModelUtil.getTransactionalEditingDomain(pSafetyArtifacts);

        RecordingCommand vCreateLocalAnalysisPackage =
            new RecordingCommand(vDomain, CREATE_LOCALANALYSIS_PACKAGE_LABEL) {

                @Override
                protected void doExecute() {
                    // Create the object 'Local Analysis' package
                    Package vLocalAnalysis = pSafetyArtifacts.createNestedPackage(LOCAL_ANALYSIS_PACKAGE_NAME);
                    // Apply 'SLocalAnalysis' stereotype on 'LocalAnalysis' package
                    UMLUtil.StereotypeApplicationHelper.getInstance(vLocalAnalysis)
                        .applyStereotype(vLocalAnalysis, ESFLocalAnalysisPackage.eINSTANCE.getSLocalAnalysis());
                }
            };

        // Verify if command can be executed
        if (vCreateLocalAnalysisPackage.canExecute()) {
            // Execute command
            vDomain.getCommandStack().execute(vCreateLocalAnalysisPackage);
        }
    }

    /**
     * Create SBlock_LA.
     *
     * @param pLocalAnalysisPackage The LocalAnalysis package
     * @param pSBlock The SBlock to be local analysed
     */
    private static void createSBlockLA(final Package pLocalAnalysisPackage, final ISBlock pSBlock) {
        TransactionalEditingDomain vDomain = ModelUtil.getTransactionalEditingDomain(pLocalAnalysisPackage);

        RecordingCommand vCreateSBlockLA = new RecordingCommand(vDomain, CREATE_SBLOCK_LA_LABEL) {

            @Override
            protected void doExecute() {
                // Create the object 'Class' of SBlock for Local Analysis
                Class vSBlockLA = pLocalAnalysisPackage.createOwnedClass(pSBlock.getName(), true);
                // Apply 'SBlockLAnalysis' stereotype on 'SBlockLA'
                UMLUtil.StereotypeApplicationHelper.getInstance(vSBlockLA)
                    .applyStereotype(vSBlockLA, ESFLocalAnalysisPackage.eINSTANCE.getSBlockLAnalysis());

                // Link 'SBlockLAnalysis' stereotype to 'SBlock' stereotype
                ISBlockLAnalysis vSBlockLAnalysis =
                    (ISBlockLAnalysis) UMLUtil.getStereotypeApplication(vSBlockLA, SBlockLAnalysis.class);
                vSBlockLAnalysis.setSBlock(pSBlock);

                // Create the ports for SBlockLA
                for (ISPort vSPort : pSBlock.getOwnedSPortsList()) {
                    Port vPortLA = vSBlockLA.createOwnedPort(vSPort.getName(), null);
                    // Apply 'SPortLAnalysis' stereotype on 'SBlockLA'
                    UMLUtil.StereotypeApplicationHelper.getInstance(vPortLA)
                        .applyStereotype(vPortLA, ESFLocalAnalysisPackage.eINSTANCE.getSPortLAnalysis());

                    // Link 'SPortLAnalysis' stereotype to 'SPort' stereotype
                    ISPortLAnalysis vSPortLAnalysis =
                        (ISPortLAnalysis) UMLUtil.getStereotypeApplication(vPortLA, SPortLAnalysis.class);
                    vSPortLAnalysis.setSPort(vSPort);
                    SDirection vSDirection = vSPort.getSDirection();
                    vSPortLAnalysis.setSDirectionLAnalysis(SDirectionLAnalysis.get(vSDirection.getValue()));

                }
            }
        };

        // Verify if command can be executed
        if (vCreateSBlockLA.canExecute()) {
            // Execute command
            vDomain.getCommandStack().execute(vCreateSBlockLA);
        }
    }

    /**
     * Apply ESFLocalAnalysis profile to the given model element.
     *
     * @param pModel The model target
     */
    private static void applyESFLocalAnalysisProfile(final Model pModel) {
        TransactionalEditingDomain vDomain = ModelUtil.getTransactionalEditingDomain(pModel);

        RecordingCommand vApplyLAProfileCmd = new RecordingCommand(vDomain, APPLY_ESFLOCALANALYSIS_PROFILE_LABEL) {

            @Override
            protected void doExecute() {
                // Retrieve ESFLocalAnalysis profile and apply it
                Profile vESFLocalAnalysisProfile = (Profile) PackageUtil
                    .loadPackage(URI.createURI(ESFLocalAnalysisSet.PROFILE_PATH), pModel.eResource().getResourceSet());

                if (vESFLocalAnalysisProfile != null) {
                    PackageUtil.applyProfile(pModel, vESFLocalAnalysisProfile, true);
                }
            }
        };

        // Verify if command can be executed
        if (vApplyLAProfileCmd.canExecute()) {
            // Execute command
            vDomain.getCommandStack().execute(vApplyLAProfileCmd);
        }
    }

}
